/**
 * @author SciLor
 */

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.*;

class SketchPanel extends JPanel implements KeyListener {
	boolean[] PressedKeys = new boolean[Short.MAX_VALUE];
	private Point direction = new Point(+1, 0);
	public int score = 0;
	private boolean playerPause = false;
	
	public SketchPanel() {
	      this.addComponentListener( new ComponentListener() {
	          public void componentResized( ComponentEvent e ) {
	             clear();
	          }
	          public void componentHidden( ComponentEvent e ) {}
	          public void componentMoved(  ComponentEvent e ) {}
	          public void componentShown(  ComponentEvent e ) {}
	       });
		addKeyListener(this);
	}

	public void keyPressed(KeyEvent evt) {
		int keyCode = evt.getKeyCode();
		PressedKeys[keyCode] = true;
	}

	public void keyReleased(KeyEvent evt) {
		int keyCode = evt.getKeyCode();
		PressedKeys[keyCode] = false;
	}

	public void keyTyped(KeyEvent evt) { }

	public void moveSketch() {
		int d;
		if (PressedKeys[KeyEvent.VK_SHIFT]) {
			d = direction.x * 5 + direction.y * 5;
		} else {
			d = direction.x * 1 + direction.y * 1;
		}
		
		score += Math.abs(d+(d-1));
		
		if (PressedKeys[KeyEvent.VK_LEFT]) {
			if (direction.x == 0) { PressedKeys[KeyEvent.VK_LEFT] = false; direction = new Point(-1, 0); }
		} else if (PressedKeys[KeyEvent.VK_RIGHT]) {
			if (direction.x == 0) { PressedKeys[KeyEvent.VK_RIGHT] = false; direction = new Point(1, 0); }
		} else if (PressedKeys[KeyEvent.VK_UP]) {
			if (direction.y == 0) { PressedKeys[KeyEvent.VK_UP] = false; direction = new Point(0, -1); }
		} else if (PressedKeys[KeyEvent.VK_DOWN]) {
			if (direction.y == 0) { PressedKeys[KeyEvent.VK_DOWN] = false; direction = new Point(0, 1); }
		} else if (PressedKeys[KeyEvent.VK_SPACE]) {
			clear();
		} else if (PressedKeys[KeyEvent.VK_ENTER]) {
			playerPause = !playerPause;
		}
			
		add(direction.x * d, direction.y * d);
	}
	
	public boolean isFocusTraversable() {
		return true;
	}

	private BufferedImage tempImage;
	
	public void add(int dx, int dy) {
		if (!playerPause) {
			for (int i=Math.abs(dx); i!=0; i-=Math.abs(direction.x)) {
				if (!addStep(direction.x,0)) {
					return;
				}
			}
			for (int i=Math.abs(dy); i!=0; i-=Math.abs(direction.y)) {
				if (!addStep(0,direction.y)) {
					return;
				}
			}
		}
		addAiStep();
		redraw();
	}
	
	private boolean addStep(int dx, int dy) {
		if (tempImage!=null && (dx!=0 || dy!=0)) { 
			end.x += dx;
			end.y += dy;
			if (end.x<0) { end.x=0; }
			if (end.y<0) { end.y=0; }
			if (end.x>=this.getWidth()) { end.x=this.getWidth()-1; }
			if (end.y>=this.getHeight()) { end.y=this.getHeight()-1; }
			
			Sketch.sf.setPoints(score);
			if (!checkCollision(end.x, end.y)) {
				Graphics g = tempImage.getGraphics();
				g.setColor(Color.white);
				g.drawLine(start.x, start.y, end.x, end.y);
				g.dispose();
				start.x = end.x;
				start.y = end.y;
			} else {
				clear();
				return false;
			}
			return true;
		} else { 
			clear();
			return false;
		}
	}
	
	private boolean checkCollision(int x, int y) {
		if (x >= 0 && y >= 0) {
			if (x < tempImage.getWidth() && y < tempImage.getHeight()) {
				if (tempImage.getRGB(x, y) == Color.black.getRGB()) {
					return false;
				}
			}
		}
		return true;
	}

	boolean[] aiAlive = new boolean[] {true, true, true};
	int[] aiColor = new int[] {Color.blue.getRGB(), Color.red.getRGB(), Color.green.getRGB()};
	int[] aiFear = new int[] {500, 200, 20};
	int[] aiThinkTime = new int[] {2, 2, 2};
	int[] aiThoughtTime = new int[] {0, 0, 0};
	int[] aiRandomDirectionChange = new int[] {3000, 2000, 1000};
	boolean[] aiCheckSight = new boolean[] {true, true, true};
	Point[] aiDirection = new Point[] { new Point(0,1), new Point(-1,0), new Point(0,-1)};
	Point[] aiPosition = new Point[] { new Point(100, 0), new Point(100, 100), new Point(0, 100)};
	
	private void addAiStep() {
		for (int i=0; i<aiColor.length; i++) {
			if (aiAlive[i]) {
				aiThoughtTime[i] +=1;
				if (aiThoughtTime[i] >= Math.random() * aiThinkTime[i]){
					aiThoughtTime[i] = 0;
					for (int j=1; j<Math.random()*aiFear[i]+1; j++) {
						if (checkCollision(aiPosition[i].x + (aiDirection[i].x * j), aiPosition[i].y + (aiDirection[i].y * j))) {						
							aiDirection[i].x = directionChange(aiDirection[i].x);
							aiDirection[i].y = directionChange(aiDirection[i].y);
							if (aiCheckSight[i]) {
								for (int k=1; j<aiFear[i]+1; k++) {
									if (checkCollision(aiPosition[i].x + (aiDirection[i].x * k), aiPosition[i].y + (aiDirection[i].y * k))) {
										aiDirection[i].x *= -1;
										aiDirection[i].y *= -1;
										break;
									}
									if (checkCollision(aiPosition[i].x + (aiDirection[i].x * k * -1), aiPosition[i].y + (aiDirection[i].y * k * -1))) {
										break;
									}
									if (k==aiFear[i]) {
										if (Math.round(Math.random()) == 1) {
											aiDirection[i].x *= -1;
											aiDirection[i].y *= -1;
										}
									}
								}
							}
							if (checkCollision(aiPosition[i].x + aiDirection[i].x, aiPosition[i].y + aiDirection[i].y)) {
								aiDirection[i].x *= -1;
								aiDirection[i].y *= -1;
							} break;
						} else {
							if (aiRandomDirectionChange[i] == (int)(Math.random() * aiRandomDirectionChange[i] + 1)) {
								aiDirection[i].x = directionChange(aiDirection[i].x);
								aiDirection[i].y = directionChange(aiDirection[i].y);
								if (checkCollision(aiPosition[i].x + aiDirection[i].x, aiPosition[i].y + aiDirection[i].y)) {
									aiDirection[i].x *= -1;
									aiDirection[i].y *= -1;
								} 
							}
						}
					}
				}
				
				aiPosition[i].x += aiDirection[i].x;
				aiPosition[i].y += aiDirection[i].y;
				if (checkCollision(aiPosition[i].x, aiPosition[i].y)) {
					aiAlive[i] = false;
				} else {
					Graphics g = tempImage.getGraphics();
					g.setColor(new Color(aiColor[i]));
					g.drawLine(aiPosition[i].x-aiDirection[i].x, aiPosition[i].y-aiDirection[i].y, aiPosition[i].x, aiPosition[i].y);
					g.dispose();
					redraw();
				}
			}
		}
	}
	
	private int directionChange(int i) {
		if (i != 0) {
			return 0;
		}
		if (Math.random()*2-1 > 0) {
			return 1;
		}
		return -1;
	}
	
	public void clear() {
		if (this.getWidth()>0 && this.getHeight()>0) {
			tempImage = new BufferedImage(this.getWidth(), this.getHeight(), BufferedImage.TYPE_INT_RGB);
			Graphics g = tempImage.getGraphics();
			g.clearRect(0, 0, this.getWidth(), this.getHeight());
			g.dispose();
			start = new Point(0, 0);
			end = new Point(0, 0);
			direction = new Point(1,0);
			
			aiAlive = new boolean[] {true, true, true};
			aiThoughtTime = new int[] {0, 0, 0};
			aiDirection = new Point[] { new Point(0,1), new Point(-1,0), new Point(0,-1)};
			aiPosition = new Point[] { new Point(this.getWidth()-1, 0), new Point(this.getWidth()-1, getHeight()-1), new Point(0, getHeight()-1)};
			
			score = 0;
			redraw();
		}
	}
	
	public void redraw() {
		Graphics g = getGraphics();
		g.drawImage(tempImage, 0, 0, null);
	}

	private Point start = new Point(0, 0);
	private Point end = new Point(0, 0);
}

class Task extends TimerTask {
	public void run() {    	
		Sketch.sf.sp.moveSketch();
	}
}


class SketchFrame extends JFrame {
	public SketchPanel sp = new SketchPanel();
	public SketchFrame() {
		setPoints(0);
		setSize(300, 200);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});

		Container contentPane = getContentPane();
		contentPane.add(sp);
	}
	public void setPoints(int score) {
		setTitle("Tron - " + score + " Points");
	}
}

public class Sketch {
	public static SketchFrame sf = new SketchFrame();
	public static Task drawTask = new Task();
	
	public static void main(String[] args) {
		JFrame frame = sf;
		Timer timer = new Timer();

		timer.schedule(new Task(), 10, 10);

		frame.setVisible(true);
	}
}
